//
//				Y. Okamoto, 2015.12
//

#ifndef CALCFDISTRI_CK
#define CALCFDISTRI_CK


#include "adapIntgrtn.h"

using namespace adapInt;

namespace calcfd {

	inline double sqr(double v){
		return v * v;
	}

	class CF0 {
	public:
		double df1, df2;
		CF0(){}
		CF0(double vdf1, double vdf2){
			df1 = vdf1;
			df2 = vdf2;
			if (df1 < 2) throw gcnew Exception("df1 should be at least 2.");
		}
		void SetDF(double vdf1, double vdf2){
			df1 = vdf1;
			df2 = vdf2;
			if (df1 < 2) throw gcnew Exception("df1 should be at least 2.");
		}

		double operator()(double x){
			if (df1 < 2) throw gcnew Exception("df1 should be at least 2.");
			double v = ((df1 / 2.0) - 1.0) * Math::Log(x)
				- ((df1 + df2) / 2.0) * Math::Log(1.0 + (df1 * x / df2));
			return Math::Exp(v);
		}
	};

	class CF0tan {
	public:
		double df1, df2;
		CF0 f0;
		CF0tan(){}
		CF0tan(double vdf1, double vdf2){
			df1 = vdf1;
			df2 = vdf2;
			f0.df1 = df1;
			f0.df2 = df2;
		}

		double operator()(double t){
			double v = f0(Math::Tan(t)) / sqr(Math::Cos(t));
			return v;
		}
	};

	class CCum_F {
	public:
		double df1, df2;
		CF0 f0;
		double coeff;
		CCum_F(){}
		CCum_F(double vdf1, double vdf2){
			df1 = vdf1;
			df2 = vdf2;
			f0.df1 = df1;
			f0.df2 = df2;
			CF0tan f0tan(df1, df2);
			TAdaptiveGL<CF0tan> gl(f0tan);
			double vg0 = gl.integral(0.0, Math::PI / 2.0);
			coeff = 1.0 / vg0;
		}

		double operator()(double F){
			TAdaptiveGL<CF0> gl(f0);
			double v = gl.integral(0.0, F);
			return v * coeff;
		}
	};


	//     Bisection@ɂ鍪̌vZ    
	template <class TF>
	void TBisection(
		TF & f,						 //	  ֐IuWFNg
		double s,					 //   f֐̂Ƃ s > 0.0
		double c,					 //   ɑ΂֐l
		double L_b, double U_b,		 //   ̑݋Ԃ̉Ə
		double & Root,				 //     f(Root) = c
		double acc){
		if ((f(L_b) - c) * (f(U_b) - c) > 0.0){
			throw  gcnew Exception("Ɖ̑gݍ킹sK؂łB");
		}
		do {
			double m = 0.5 * (L_b + U_b);
			double v = f(m);
			if (s * (v - c) > 0.0) {
				U_b = m;
			}
			else {
				L_b = m;
			}
		} while ((Math::Abs(U_b - L_b) >= acc * Math::Abs(U_b))
			&&
			((Math::Abs(U_b) + Math::Abs(L_b)) >= 1.0e-14)
			);
		Root = 0.5 * (L_b + U_b);
	}


}

#endif
